/*
 * Ukazkovy program pro 2. cv. ZPG
 * 
 * Jiri Skala, 2006
 * 
 */
using System;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace zpg_cv02
{
    class ZpgD3D : Form
    {
        /// <summary>
        /// Direct3D device.
        /// </summary>
        private Device device = null;

        /// <summary>
        /// Direct3D vertx buffer.
        /// </summary>
        private VertexBuffer vertexBuff = null;



        /// <summary>
        /// Constructor.
        /// </summary>
        public ZpgD3D()
        {
            // set window properties
            this.Size = new Size(640, 480);
            this.Text = "Direct3D example";
        }

        /// <summary>
        /// Direct3D initialization.
        /// </summary>
        /// <returns></returns>
        private bool InitD3D()
        {
            try
            {
                // prepare presentation parameters
                PresentParameters presParams = new PresentParameters();
                presParams.Windowed = true;
                presParams.SwapEffect = SwapEffect.Discard;
                presParams.BackBufferFormat = Format.Unknown;
                presParams.EnableAutoDepthStencil = false;

                // determine graphic card capabilities
                Caps caps = Manager.GetDeviceCaps(0, DeviceType.Hardware);
                CreateFlags flags;
                if (caps.DeviceCaps.SupportsHardwareTransformAndLight)
                    flags = CreateFlags.HardwareVertexProcessing;
                else
                    flags = CreateFlags.SoftwareVertexProcessing;

                // create D3D device
                device = new Device(0, DeviceType.Hardware, this, flags, presParams);

                // create vertex buffer
                vertexBuff = new VertexBuffer(typeof(CustomVertex.PositionOnly), 10, device,
                    Usage.WriteOnly, CustomVertex.PositionOnly.Format, Pool.Managed);
                // fill the buffer with vertices
                CustomVertex.PositionOnly[] verts
                    = (CustomVertex.PositionOnly[])vertexBuff.Lock(0, LockFlags.None);
                verts[0] = new CustomVertex.PositionOnly(-1, -1, -1);
                verts[1] = new CustomVertex.PositionOnly( 1, -1, -1);
                verts[2] = new CustomVertex.PositionOnly(-1,  1, -1);
                verts[3] = new CustomVertex.PositionOnly( 1,  1, -1);
                verts[4] = new CustomVertex.PositionOnly(-1,  1,  1);
                verts[5] = new CustomVertex.PositionOnly( 1,  1,  1);
                verts[6] = new CustomVertex.PositionOnly(-1, -1,  1);
                verts[7] = new CustomVertex.PositionOnly( 1, -1,  1);
                verts[8] = new CustomVertex.PositionOnly(-1, -1, -1);
                verts[9] = new CustomVertex.PositionOnly( 1, -1, -1);
                vertexBuff.Unlock();

                return true;
            }
            catch (DirectXException)
            {
                // something is wrong
                MessageBox.Show("Direct3D initialization failed.", "Error",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
        }

        /// <summary>
        /// Render the scene.
        /// </summary>
        private void Render()
        {
            if (device == null)
                return;

            // set rendering properties
            device.RenderState.Lighting = false;
            device.RenderState.CullMode = Cull.None;

            // set up transformations
            float aspect = (float)this.ClientSize.Width / (float)this.ClientSize.Height;
            device.Transform.Projection = Matrix.PerspectiveFovLH((float)Math.PI / 4, aspect, 0.1f, 10.0f);

            device.Transform.View = Matrix.Translation(0, 0, 5);

            // clear scene
            device.Clear(ClearFlags.Target, Color.DarkBlue, 1, 0);
            // start rendering
            device.BeginScene();

            device.SetStreamSource(0, vertexBuff, 0);
            device.VertexFormat = CustomVertex.PositionOnly.Format;
            // draw the cube
            device.DrawPrimitives(PrimitiveType.TriangleStrip, 0, 8);

            // rendering done
            device.EndScene();
            // show result
            device.Present();
        }

        /// <summary>
        /// Main "game" loop.
        /// </summary>
        private void Run()
        {
            // enter "game loop"
            while (this.Created)
            {
                Render();
                Application.DoEvents(); // should be the last in this loop
            }

            // dispose Direct3D resources
            if (vertexBuff != null)
                vertexBuff.Dispose();
            if (device != null)
                device.Dispose();
        }

        /// <summary>
        /// Application entry point.
        /// </summary>
        static void Main(string[] args)
        {
            // create application window
            ZpgD3D form = new ZpgD3D();
            // make sure it's visible
            form.Show();

            // initialize Direct3D
            if (form.InitD3D())
            {
                form.Run();
            }
        }
    }
}
